@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003-2006, 2008-2009, 2011-2012, 2014
+  Copyright (C) 2003-2006, 2008-2009, 2011-2012, 2014, 2017
   Rocky Bernstein <rocky@gnu.org>
   Copyright (C) 2001, 2003 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -247,16 +247,15 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
 	  entries /= sizeof (_cuex_array_t);
 
 	  if (CUES_ID == opcode) {
-	    lsn_t lsn = UINT32_FROM_BE (_entries[0].lsn);
 	    unsigned int idx;
 	    unsigned int i = 0;
-
-	    cdio_debug ("CUES type image detected" );
-
+	    lsn_t lsn; /* = UINT32_FROM_BE (_entries[0].lsn); */
 	    /* CUES LSN has 150 pregap include at beginning? -/
 	       cdio_assert (lsn == 0?);
 	    */
 
+	    cdio_debug ("CUES type image detected" );
+
 	    p_env->is_cues           = true; /* HACK alert. */
 	    p_env->gen.i_tracks      = 0;
 	    p_env->gen.i_first_track = 1;
@@ -1107,7 +1106,7 @@ _free_nrg (void *p_user_data)
 
   if (NULL == p_env) return;
   if (NULL != p_env->mapping)
-    _cdio_list_free (p_env->mapping, true);
+    _cdio_list_free (p_env->mapping, true, free);
 
   /* The remaining part of the image is like the other image drivers,
      so free that in the same way. */
@@ -1268,29 +1267,36 @@ _get_track_green_nrg(void *p_user_data, track_t track_num)
 bool
 cdio_is_nrg(const char *psz_nrg)
 {
-  _img_private_t env;
+  _img_private_t *p_env  = calloc(1, sizeof (_img_private_t));
   bool is_nrg = false;
 
-  if (psz_nrg == NULL) return false;
-
-  memset(&env, 0, sizeof(env));
-  if (!(env.gen.data_source = cdio_stdio_new (psz_nrg))) {
-    cdio_warn ("can't open nrg image file %s for reading", psz_nrg);
-    return false;
+  if (psz_nrg == NULL) {
+    is_nrg = false;
+    goto exit;
   }
 
-  if (parse_nrg(&env, psz_nrg, CDIO_LOG_INFO)) {
+  if (!(p_env->gen.data_source = cdio_stdio_new (psz_nrg))) {
+    cdio_warn ("can't open nrg image file %s for reading", psz_nrg);
+    is_nrg = false;
+    goto exit;
+  }
+
+  if (parse_nrg(p_env, psz_nrg, CDIO_LOG_INFO)) {
     is_nrg = true;
 #ifdef ALSO_TEST_NAME
     size_t psz_len;
     psz_len = strlen(psz_nrg);
     /* At least 4 characters needed for .nrg extension */
-    if ( psz_len < 4 ) return false;
+    if ( psz_len < 4 ) {
+      is_nrg = false;
+      goto exit;
+    }
 
     is_nrg = strncasecmp( psz_nrg+(psz_len-3), "nrg", 3 ) == 0;
 #endif
   }
-  cdio_stdio_destroy(env.gen.data_source);
+ exit:
+  _free_nrg(p_env);
   return is_nrg;
 }
 
